@@ -1,17 +1,12 @@
-// Copyright (C) 2013-2014  Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
 //
-// Permission to use, copy, modify, and/or distribute this software for any
-// purpose with or without fee is hereby granted, provided that the above
-// copyright notice and this permission notice appear in all copies.
-//
-// THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
-// REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
-// AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
-// INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
-// OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-// PERFORMANCE OF THIS SOFTWARE.
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+#include <asiolink/asio_wrapper.h>
+#include <asiolink/io_service.h>
 #include <asiolink/interval_timer.h>
 #include <d2/nc_trans.h>
 #include <dns/opcode.h>
@@ -21,11 +16,6 @@
 #include <util/buffer.h>
 #include <nc_test_utils.h>
 
-#include <asio/ip/udp.hpp>
-#include <asio/socket_base.hpp>
-#include <asio.hpp>
-#include <asio/error_code.hpp>
-
 #include <boost/function.hpp>
 #include <boost/bind.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
@@ -34,6 +24,7 @@
 using namespace std;
 using namespace isc;
 using namespace isc::d2;
+using namespace isc::util;
 using namespace boost::posix_time;
 
 namespace {
@@ -59,7 +50,7 @@ public:
     /// @brief Constructor
     ///
     /// Parameters match those needed by NameChangeTransaction.
-    NameChangeStub(IOServicePtr& io_service,
+    NameChangeStub(asiolink::IOServicePtr& io_service,
                    dhcp_ddns::NameChangeRequestPtr& ncr,
                    DdnsDomainPtr& forward_domain,
                    DdnsDomainPtr& reverse_domain,
@@ -74,8 +65,8 @@ public:
     }
 
     /// @brief DNSClient callback
-    /// Overrides the callback in NameChangeTranscation to allow testing
-    /// sendUpdate without incorporating exectution of the state model
+    /// Overrides the callback in NameChangeTransaction to allow testing
+    /// sendUpdate without incorporating execution of the state model
     /// into the test.
     /// It sets the DNS status update and posts IO_COMPLETED_EVT as does
     /// the normal callback.
@@ -96,7 +87,7 @@ public:
         if (getForwardDomain()) {
             initServerSelection(getForwardDomain());
             selectNextServer();
-            return (getCurrentServer());
+            return (getCurrentServer().get() != 0);
         }
 
         return (false);
@@ -230,7 +221,7 @@ public:
         NameChangeTransaction::verifyStates();
 
         // Check our states.
-        getState(DOING_UPDATE_ST);
+        getStateInternal(DOING_UPDATE_ST);
     }
 
     // Expose the protected methods to be tested.
@@ -322,7 +313,7 @@ public:
     /// @brief Builds and then sends an update request
     ///
     /// This method is used to build and send and update request. It is used
-    /// in conjuction with FauxServer to test various message response
+    /// in conjunction with FauxServer to test various message response
     /// scenarios.
     /// @param name_change Transaction under test
     /// @param run_time Maximum time to permit IO processing to run before
@@ -367,19 +358,19 @@ public:
 /// requires reverse change.
 /// 4. Valid construction functions properly
 TEST(NameChangeTransaction, construction) {
-    IOServicePtr io_service(new isc::asiolink::IOService());
+    asiolink::IOServicePtr io_service(new isc::asiolink::IOService());
     D2CfgMgrPtr cfg_mgr(new D2CfgMgr());
 
     const char* msg_str =
         "{"
-        " \"change_type\" : 0 , "
-        " \"forward_change\" : true , "
-        " \"reverse_change\" : true , "
+        " \"change-type\" : 0 , "
+        " \"forward-change\" : true , "
+        " \"reverse-change\" : true , "
         " \"fqdn\" : \"example.com.\" , "
-        " \"ip_address\" : \"192.168.2.1\" , "
+        " \"ip-address\" : \"192.168.2.1\" , "
         " \"dhcid\" : \"0102030405060708\" , "
-        " \"lease_expires_on\" : \"20130121132405\" , "
-        " \"lease_length\" : 1300 "
+        " \"lease-expires-on\" : \"20130121132405\" , "
+        " \"lease-length\" : 1300 "
         "}";
 
     dhcp_ddns::NameChangeRequestPtr ncr;
@@ -396,7 +387,7 @@ TEST(NameChangeTransaction, construction) {
 
     // Verify that construction with a null IOServicePtr fails.
     // @todo Subject to change if multi-threading is implemented.
-    IOServicePtr empty;
+    asiolink::IOServicePtr empty;
     EXPECT_THROW(NameChangeTransaction(empty, ncr,
                                        forward_domain, reverse_domain, cfg_mgr),
                                        NameChangeTransactionError);
@@ -449,7 +440,7 @@ TEST(NameChangeTransaction, construction) {
 }
 
 /// @brief General testing of member accessors.
-/// Most if not all of these are also tested as a byproduct off larger tests.
+/// Most if not all of these are also tested as a byproduct of larger tests.
 TEST_F(NameChangeTransactionTest, accessors) {
     NameChangeStubPtr name_change;
     ASSERT_NO_THROW(name_change = makeCannedTransaction());
@@ -461,6 +452,9 @@ TEST_F(NameChangeTransactionTest, accessors) {
     // Verify that getTransactionKey works.
     EXPECT_EQ(ncr->getDhcid(), name_change->getTransactionKey());
 
+    // Verify that getRequestId works.
+    EXPECT_EQ(ncr->getRequestId(), name_change->getRequestId());
+
     // Verify that NcrStatus can be set and retrieved.
     EXPECT_NO_THROW(name_change->setNcrStatus(dhcp_ddns::ST_FAILED));
     EXPECT_EQ(dhcp_ddns::ST_FAILED, ncr->getStatus());
@@ -830,7 +824,7 @@ TEST_F(NameChangeTransactionTest, successfulUpdateTest) {
     EXPECT_TRUE(name_change->getForwardChangeCompleted());
 }
 
-/// @brief Tests the ability to use startTransaction to initate the state
+/// @brief Tests the ability to use startTransaction to initiate the state
 /// model execution, and DNSClient callback, operator(), to resume the
 /// model with a update failure outcome.
 TEST_F(NameChangeTransactionTest, failedUpdateTest) {
@@ -842,7 +836,7 @@ TEST_F(NameChangeTransactionTest, failedUpdateTest) {
     // should run up until the "IO" operation is initiated in DOING_UPDATE_ST.
     ASSERT_NO_THROW(name_change->startTransaction());
 
-    // Vefity that the model is running but waiting, and that the forward
+    // Verify that the model is running but waiting, and that the forward
     // change has not been completed.
     EXPECT_TRUE(name_change->isModelRunning());
     EXPECT_TRUE(name_change->isModelWaiting());
@@ -1049,7 +1043,7 @@ TEST_F(NameChangeTransactionTest, tsigUnsignedResponse) {
     FauxServer server(*io_service_, *(name_change->getCurrentServer()));
     server.receive (FauxServer::USE_RCODE, dns::Rcode::NOERROR());
 
-    // Do the udpate.
+    // Do the update.
     ASSERT_NO_FATAL_FAILURE(doOneExchange(name_change));
 
     // Verify that next event is IO_COMPLETED_EVT and DNS status is
@@ -1080,7 +1074,7 @@ TEST_F(NameChangeTransactionTest, tsigInvalidResponse) {
     FauxServer server(*io_service_, *(name_change->getCurrentServer()));
     server.receive (FauxServer::INVALID_TSIG, dns::Rcode::NOERROR());
 
-    // Do the udpate.
+    // Do the update.
     ASSERT_NO_FATAL_FAILURE(doOneExchange(name_change));
 
     // Verify that next event is IO_COMPLETED_EVT and DNS status is
@@ -1130,7 +1124,7 @@ TEST_F(NameChangeTransactionTest, tsigUnexpectedSignedResponse) {
     EXPECT_EQ("response.example.com.", zone->getName().toText());
 }
 
-/// @brief Tests that a TSIG udpate succeeds when client and server both use
+/// @brief Tests that a TSIG update succeeds when client and server both use
 /// the right key.  Runs the test for all supported algorithms.
 TEST_F(NameChangeTransactionTest, tsigAllValid) {
     std::vector<std::string>algorithms;
@@ -1201,6 +1195,15 @@ TEST_F(NameChangeTransactionTest, prepNewRequest) {
     // valid input.
     ASSERT_NO_THROW(request = name_change->prepNewRequest(forward_domain_));
     checkZone(request, forward_domain_->getName());
+
+    // The query id is random so 0 is not impossible
+    for (unsigned i = 0; i < 10; ++i) {
+        if (request->getId() == 0) {
+            request = name_change->prepNewRequest(forward_domain_);
+        }
+    }
+
+    EXPECT_NE(0, request->getId());
 }
 
 /// @brief Tests the addLeaseAddressRData method
