@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -433,7 +433,9 @@ Dhcpv6SrvTest::testRenewBasic(Lease::Type type,
     // equality or difference by 1 between cltt and expected is ok.
     EXPECT_GE(1, abs(cltt - expected));
 
-    EXPECT_TRUE(LeaseMgrFactory::instance().deleteLease(renew_addr));
+    Lease6Ptr lease(new Lease6());
+    lease->addr_ = renew_addr;
+    EXPECT_TRUE(LeaseMgrFactory::instance().deleteLease(lease));
 }
 
 void
@@ -755,7 +757,9 @@ Dhcpv6SrvTest::testReleaseReject(Lease::Type type, const IOAddress& addr) {
     EXPECT_EQ(1, stat->getInteger().first);
 
     // Finally, let's cleanup the database
-    EXPECT_TRUE(LeaseMgrFactory::instance().deleteLease(addr));
+    lease.reset(new Lease6());
+    lease->addr_ = addr;
+    EXPECT_TRUE(LeaseMgrFactory::instance().deleteLease(lease));
 }
 
 void
@@ -964,6 +968,6 @@ NakedDhcpv6SrvTest::checkIA_NAStatusCode(
     }
 }
 
-}; // end of isc::dhcp::test namespace
-}; // end of isc::dhcp namespace
-}; // end of isc namespace
+}  // namespace test
+}  // namespace dhcp
+}  // namespace isc
