@@ -321,6 +321,11 @@ TEST_F(HAImplTest, leases4Committed) {
     ASSERT_NO_THROW(ha_impl.startService(io_service_, network_state,
                                          HAServerType::DHCPv4));
 
+    // Make sure we wait for the acks from the backup server to be able to
+    // test the case of sending lease updates even though the service is
+    // in the state in which the lease updates are normally not sent.
+    ha_impl.config_->setWaitBackupAck(true);
+
     // Create callout handle to be used for passing arguments to the
     // callout.
     CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
@@ -388,6 +393,11 @@ TEST_F(HAImplTest, leases6Committed) {
     ASSERT_NO_THROW(ha_impl.startService(io_service_, network_state,
                                          HAServerType::DHCPv6));
 
+    // Make sure we wait for the acks from the backup server to be able to
+    // test the case of sending lease updates even though the service is
+    // in the state in which the lease updates are normally not sent.
+    ha_impl.config_->setWaitBackupAck(true);
+
     // Create callout handle to be used for passing arguments to the
     // callout.
     CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
@@ -563,20 +573,126 @@ TEST_F(HAImplTest, statusGet) {
     std::string expected =
         "{"
         "    \"arguments\": {"
-        "        \"ha-servers\": {"
-        "            \"local\": {"
-        "                \"role\": \"primary\","
-        "                \"scopes\": [  ],"
-        "                \"state\": \"waiting\""
-        "            },"
-        "            \"remote\": {"
-        "                \"age\": 0,"
-        "                \"in-touch\": false,"
-        "                \"last-scopes\": [ ],"
-        "                \"last-state\": \"\","
-        "                \"role\": \"secondary\""
+        "        \"high-availability\": ["
+        "            {"
+        "                \"ha-mode\": \"load-balancing\","
+        "                \"ha-servers\": {"
+        "                    \"local\": {"
+        "                        \"role\": \"primary\","
+        "                        \"scopes\": [  ],"
+        "                        \"state\": \"waiting\""
+        "                    },"
+        "                    \"remote\": {"
+        "                        \"age\": 0,"
+        "                        \"in-touch\": false,"
+        "                        \"last-scopes\": [ ],"
+        "                        \"last-state\": \"\","
+        "                        \"role\": \"secondary\","
+        "                        \"communication-interrupted\": false,"
+        "                        \"connecting-clients\": 0,"
+        "                        \"unacked-clients\": 0,"
+        "                        \"unacked-clients-left\": 0,"
+        "                        \"analyzed-packets\": 0"
+        "                    }"
+        "                }"
         "            }"
-        "        },"
+        "        ],"
+        "        \"pid\": 1"
+        "    },"
+        "    \"result\": 0"
+        "}";
+    EXPECT_TRUE(isEquivalent(got, Element::fromJSON(expected)));
+}
+
+// Tests status-get command processed handler for backup server.
+TEST_F(HAImplTest, statusGetBackupServer) {
+    TestHAImpl ha_impl;
+    ASSERT_NO_THROW(ha_impl.configure(createValidJsonConfiguration()));
+    ha_impl.config_->setThisServerName("server3");
+
+    // Starting the service is required prior to running any callouts.
+    NetworkStatePtr network_state(new NetworkState(NetworkState::DHCPv4));
+    ASSERT_NO_THROW(ha_impl.startService(io_service_, network_state,
+                                         HAServerType::DHCPv4));
+
+    std::string name = "status-get";
+    ConstElementPtr response =
+        Element::fromJSON("{ \"arguments\": { \"pid\": 1 }, \"result\": 0 }");
+
+    CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
+
+    callout_handle->setArgument("name", name);
+    callout_handle->setArgument("response", response);
+
+    ASSERT_NO_THROW(ha_impl.commandProcessed(*callout_handle));
+
+    ConstElementPtr got;
+    callout_handle->getArgument("response", got);
+    ASSERT_TRUE(got);
+
+    std::string expected =
+        "{"
+        "    \"arguments\": {"
+        "        \"high-availability\": ["
+        "            {"
+        "                \"ha-mode\": \"load-balancing\","
+        "                \"ha-servers\": {"
+        "                    \"local\": {"
+        "                        \"role\": \"backup\","
+        "                        \"scopes\": [  ],"
+        "                        \"state\": \"backup\""
+        "                    }"
+        "                }"
+        "            }"
+        "        ],"
+        "        \"pid\": 1"
+        "    },"
+        "    \"result\": 0"
+        "}";
+    EXPECT_TRUE(isEquivalent(got, Element::fromJSON(expected)));
+}
+
+// Tests status-get command processed handler for primary server being in the
+// passive-backup state.
+TEST_F(HAImplTest, statusGetPassiveBackup) {
+    TestHAImpl ha_impl;
+    ASSERT_NO_THROW(ha_impl.configure(createValidPassiveBackupJsonConfiguration()));
+
+    // Starting the service is required prior to running any callouts.
+    NetworkStatePtr network_state(new NetworkState(NetworkState::DHCPv4));
+    ASSERT_NO_THROW(ha_impl.startService(io_service_, network_state,
+                                         HAServerType::DHCPv4));
+
+    std::string name = "status-get";
+    ConstElementPtr response =
+        Element::fromJSON("{ \"arguments\": { \"pid\": 1 }, \"result\": 0 }");
+
+    CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
+
+    callout_handle->setArgument("name", name);
+    callout_handle->setArgument("response", response);
+
+    ASSERT_NO_THROW(ha_impl.commandProcessed(*callout_handle));
+
+    ConstElementPtr got;
+    callout_handle->getArgument("response", got);
+    ASSERT_TRUE(got);
+
+    std::string expected =
+        "{"
+        "    \"arguments\": {"
+        "        \"high-availability\": ["
+        "            {"
+        "                \"ha-mode\": \"passive-backup\","
+        "                \"ha-servers\": {"
+        "                    \"local\": {"
+        "                        \"role\": \"primary\","
+        "                        \"scopes\": [ \"server1\" ],"
+        "                        \"state\": \"passive-backup\""
+        "                    }"
+        "                }"
+        "            }"
+        "        ],"
         "        \"pid\": 1"
         "    },"
         "    \"result\": 0"
