@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,14 +11,18 @@
 #include <dhcp/option.h>
 #include <dhcp/option_custom.h>
 #include <dhcp/option6_ia.h>
+#include <dhcp/option6_iaaddr.h>
+#include <dhcp/option6_iaprefix.h>
 #include <dhcp/option_int.h>
 #include <dhcp/option_int_array.h>
+#include <dhcp/option_string.h>
 #include <dhcp/option_vendor.h>
 #include <dhcp/iface_mgr.h>
 #include <dhcp/pkt6.h>
 #include <dhcp/hwaddr.h>
 #include <dhcp/docsis3_option_defs.h>
 #include <dhcp/tests/pkt_captures.h>
+#include <testutils/gtest_utils.h>
 #include <util/range_utilities.h>
 #include <boost/bind.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
@@ -410,14 +414,14 @@ TEST_F(Pkt6Test, unpackVendorMalformed) {
     shorth.resize(orig.size() - 4);
     shorth[len_index] = 12;
     Pkt6Ptr too_short_header_pkt(new Pkt6(&shorth[0], shorth.size()));
-    EXPECT_THROW(too_short_header_pkt->unpack(), OutOfRange);
+    EXPECT_THROW(too_short_header_pkt->unpack(), SkipRemainingOptionsError);
 
     // Truncated option data is not accepted
     vector<uint8_t> shorto = orig;
     shorto.resize(orig.size() - 2);
     shorto[len_index] = 16;
     Pkt6Ptr too_short_option_pkt(new Pkt6(&shorto[0], shorto.size()));
-    EXPECT_THROW(too_short_option_pkt->unpack(), OutOfRange);
+    EXPECT_THROW(too_short_option_pkt->unpack(), SkipRemainingOptionsError);
 }
 
 // This test verifies that options can be added (addOption()), retrieved
@@ -1054,7 +1058,7 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
 TEST_F(Pkt6Test, toText) {
 
     // This packet contains doubly relayed solicit. The inner-most
-    // relay-forward contains inteface-id and remote-id. We will
+    // relay-forward contains interface-id and remote-id. We will
     // check that these are printed correctly.
     Pkt6Ptr msg(capture2());
     EXPECT_NO_THROW(msg->unpack());
@@ -1089,13 +1093,13 @@ TEST_F(Pkt6Test, clientClasses) {
     // Default values (do not belong to any class)
     EXPECT_FALSE(pkt.inClass(DOCSIS3_CLASS_EROUTER));
     EXPECT_FALSE(pkt.inClass(DOCSIS3_CLASS_MODEM));
-    EXPECT_TRUE(pkt.classes_.empty());
+    EXPECT_TRUE(pkt.getClasses().empty());
 
     // Add to the first class
     pkt.addClass(DOCSIS3_CLASS_EROUTER);
     EXPECT_TRUE(pkt.inClass(DOCSIS3_CLASS_EROUTER));
     EXPECT_FALSE(pkt.inClass(DOCSIS3_CLASS_MODEM));
-    ASSERT_FALSE(pkt.classes_.empty());
+    ASSERT_FALSE(pkt.getClasses().empty());
 
     // Add to a second class
     pkt.addClass(DOCSIS3_CLASS_MODEM);
@@ -1111,6 +1115,34 @@ TEST_F(Pkt6Test, clientClasses) {
     EXPECT_TRUE(pkt.inClass("foo"));
 }
 
+// Tests whether a packet can be marked to evaluate later a class and
+// after check if a given class is in the collection
+TEST_F(Pkt6Test, deferredClientClasses) {
+    Pkt6 pkt(DHCPV6_ADVERTISE, 1234);
+
+    // Default values (do not belong to any class)
+    EXPECT_TRUE(pkt.getClasses(true).empty());
+
+    // Add to the first class
+    pkt.addClass(DOCSIS3_CLASS_EROUTER, true);
+    EXPECT_EQ(1, pkt.getClasses(true).size());
+
+    // Add to a second class
+    pkt.addClass(DOCSIS3_CLASS_MODEM, true);
+    EXPECT_EQ(2, pkt.getClasses(true).size());
+    EXPECT_TRUE(pkt.getClasses(true).contains(DOCSIS3_CLASS_EROUTER));
+    EXPECT_TRUE(pkt.getClasses(true).contains(DOCSIS3_CLASS_MODEM));
+    EXPECT_FALSE(pkt.getClasses(true).contains("foo"));
+
+    // Check that it's ok to add to the same class repeatedly
+    EXPECT_NO_THROW(pkt.addClass("foo", true));
+    EXPECT_NO_THROW(pkt.addClass("foo", true));
+    EXPECT_NO_THROW(pkt.addClass("foo", true));
+
+    // Check that the packet belongs to 'foo'
+    EXPECT_TRUE(pkt.getClasses(true).contains("foo"));
+}
+
 // Tests whether MAC can be obtained and that MAC sources are not
 // confused.
 TEST_F(Pkt6Test, getMAC) {
@@ -1553,7 +1585,7 @@ TEST_F(Pkt6Test, getMACFromRemoteIdRelayOption) {
 // (Relay Supplied Options option). This test checks whether it was parsed
 // properly. See captureRelayed2xRSOO() description for details.
 TEST_F(Pkt6Test, rsoo) {
-    Pkt6Ptr msg = test::PktCaptures::captureRelayed2xRSOO();
+    Pkt6Ptr msg = dhcp::test::PktCaptures::captureRelayed2xRSOO();
 
     EXPECT_NO_THROW(msg->unpack());
 
@@ -1689,9 +1721,341 @@ TEST_F(Pkt6Test, getLabelEmptyClientId) {
     // Create a packet.
     Pkt6 pkt(DHCPV6_SOLICIT, 0x2312);
 
-    // Add empty client idenitifier option.
+    // Add empty client identifier option.
     pkt.addOption(OptionPtr(new Option(Option::V6, D6O_CLIENTID)));
     EXPECT_EQ("duid=[no info], tid=0x2312", pkt.getLabel());
 }
 
+// Verifies that when the VIVSO, 17, has length that is too
+// short (i.e. less than sizeof(uint8_t), unpack throws a
+// SkipRemainingOptionsError exception
+TEST_F(Pkt6Test, truncatedVendorLength) {
+
+    // Build a good Solicit packet
+    Pkt6Ptr pkt = dhcp::test::PktCaptures::captureSolicitWithVIVSO();
+
+    // Unpacking should not throw
+    ASSERT_NO_THROW(pkt->unpack());
+    ASSERT_EQ(DHCPV6_SOLICIT, pkt->getType());
+
+    // VIVSO option should be there
+    OptionPtr x = pkt->getOption(D6O_VENDOR_OPTS);
+    ASSERT_TRUE(x);
+    ASSERT_EQ(D6O_VENDOR_OPTS, x->getType());
+    OptionVendorPtr vivso = boost::dynamic_pointer_cast<OptionVendor>(x);
+    ASSERT_TRUE(vivso);
+    EXPECT_EQ(8, vivso->len()); // data + opt code + len
+
+    // Build a bad Solicit packet
+    pkt = dhcp::test::PktCaptures::captureSolicitWithTruncatedVIVSO();
+
+    // Unpack should throw Skip exception
+    ASSERT_THROW(pkt->unpack(), SkipRemainingOptionsError);
+    ASSERT_EQ(DHCPV6_SOLICIT, pkt->getType());
+
+    // VIVSO option should not be there
+    x = pkt->getOption(D6O_VENDOR_OPTS);
+    ASSERT_FALSE(x);
+}
+
+// Checks that unpacking correctly handles SkipThisOptionError by
+// omitting the offending option from the unpacked options.
+TEST_F(Pkt6Test, testSkipThisOptionError) {
+    // Get a packet. We're really interested in its on-wire
+    // representation only.
+    Pkt6Ptr donor(capture1());
+
+    // That's our original content. It should be sane.
+    OptionBuffer orig = donor->data_;
+
+    orig.push_back(0);
+    orig.push_back(41);   // new-posix-timezone
+    orig.push_back(0);
+    orig.push_back(3);    // length=3
+    orig.push_back(0x61); // data="abc"
+    orig.push_back(0x62);
+    orig.push_back(0x63);
+
+    orig.push_back(0);
+    orig.push_back(59);   // bootfile-url
+    orig.push_back(0);
+    orig.push_back(3);    // length=3
+    orig.push_back(0);    // data= all nulls
+    orig.push_back(0);
+    orig.push_back(0);
+
+    orig.push_back(0);
+    orig.push_back(42);   // new-tzdb-timezone
+    orig.push_back(0);
+    orig.push_back(3);    // length=3
+    orig.push_back(0x64); // data="def"
+    orig.push_back(0x65);
+    orig.push_back(0x66);
+
+    // Unpacking should not throw.
+    Pkt6Ptr pkt(new Pkt6(&orig[0], orig.size()));
+    ASSERT_NO_THROW_LOG(pkt->unpack());
+
+    // We should have option 41 = "abc".
+    OptionPtr opt;
+    OptionStringPtr opstr;
+    ASSERT_TRUE(opt = pkt->getOption(41));
+    ASSERT_TRUE(opstr = boost::dynamic_pointer_cast<OptionString>(opt));
+    EXPECT_EQ(3, opstr->getValue().length());
+    EXPECT_EQ("abc", opstr->getValue());
+
+    // We should not have option 59.
+    EXPECT_FALSE(opt = pkt->getOption(59));
+
+    // We should have option 42 = "def".
+    ASSERT_TRUE(opt = pkt->getOption(42));
+    ASSERT_TRUE(opstr = boost::dynamic_pointer_cast<OptionString>(opt));
+    EXPECT_EQ(3, opstr->getValue().length());
+    EXPECT_EQ("def", opstr->getValue());
+}
+
+// This test verifies that LQ_QUERY_OPTIONs can be created, packed,
+// and unpacked correctly.
+TEST_F(Pkt6Test, lqQueryOption) {
+
+    OptionDefinitionPtr def = LibDHCP::getOptionDef(DHCP6_OPTION_SPACE, D6O_LQ_QUERY);
+    ASSERT_TRUE(def) << "D6O_LQ_QUERY is not undefined";
+
+    OptionCustomPtr lq_option(new OptionCustom(*def, Option::V6));
+    ASSERT_TRUE(lq_option);
+
+    // Add query type (77 is technically not valid but better visually).
+    uint8_t orig_type = 77;
+    ASSERT_NO_THROW_LOG(lq_option->writeInteger<uint8_t>(77,0));
+
+    // Add query link address
+    IOAddress orig_link("2001:db8::1");
+    ASSERT_NO_THROW_LOG(lq_option->writeAddress(orig_link, 1));
+
+    // Now add supported sub-options: D6O_IAADR, D6O_CLIENTID, and D6O_ORO
+    // We are ingoring the fact that a query containing both a D6O_IAADDR
+    // and a D6O_CLIENTID is not technically valid.  We only care that the
+    // sub options will pack and unpack.
+
+    // Add a D6O_IAADDR option
+    Option6IAAddrPtr orig_iaaddr(new Option6IAAddr(D6O_IAADDR, IOAddress("2001:db8::2"), 0, 0));
+    ASSERT_TRUE(orig_iaaddr);
+    ASSERT_NO_THROW_LOG(lq_option->addOption(orig_iaaddr));
+
+    // Add a D6O_CLIENTID option
+    DuidPtr duid(new DUID(DUID::fromText("0102020202030303030303")));
+    OptionPtr orig_clientid(new Option(Option::V6, D6O_CLIENTID, OptionBuffer(
+                                       duid->getDuid().begin(), duid->getDuid().end())));
+    ASSERT_NO_THROW_LOG(lq_option->addOption(orig_clientid));
+
+    // Add a D6O_ORO option
+    OptionUint16ArrayPtr orig_oro(new OptionUint16Array(Option::V6, D6O_ORO));
+    ASSERT_TRUE(orig_oro);
+    orig_oro->addValue(1234);
+    ASSERT_NO_THROW_LOG(lq_option->addOption(orig_oro));
+
+    // Now let's create a packet to which to add our new lq_option.
+    Pkt6Ptr orig(new Pkt6(DHCPV6_LEASEQUERY, 0x2312));
+    orig->addOption(lq_option);
+    ASSERT_NO_THROW_LOG(orig->pack());
+
+    // Now create second packet,based on assembled data from the first one
+    Pkt6Ptr clone(new Pkt6(static_cast<const uint8_t*>
+                           (orig->getBuffer().getData()),
+                            orig->getBuffer().getLength()));
+
+    // Unpack it.
+    ASSERT_NO_THROW_LOG(clone->unpack());
+
+    // We should be able to find our query option.
+    OptionPtr opt;
+    opt = clone->getOption(D6O_LQ_QUERY);
+    ASSERT_TRUE(opt);
+    OptionCustomPtr clone_query = boost::dynamic_pointer_cast<OptionCustom>(opt);
+    ASSERT_TRUE(clone_query);
+
+    // Verify the query type is right.
+    uint8_t clone_type;
+    ASSERT_NO_THROW_LOG(clone_type = clone_query->readInteger<uint8_t>(0));
+    EXPECT_EQ(orig_type, clone_type);
+
+    // Verify the query link address is right.
+    IOAddress clone_link("::");
+    ASSERT_NO_THROW_LOG(clone_link = clone_query->readAddress(1));
+    EXPECT_EQ(orig_link, clone_link);
+
+    // Verify the suboptions.
+
+    // Verify the D6O_IAADDR option
+    opt = clone_query->getOption(D6O_IAADDR);
+    ASSERT_TRUE(opt);
+    Option6IAAddrPtr clone_iaaddr = boost::dynamic_pointer_cast<Option6IAAddr>(opt);
+    ASSERT_TRUE(clone_iaaddr);
+    EXPECT_TRUE(clone_iaaddr->equals(*orig_iaaddr));
+
+    // Verify the D6O_CLIENTID option
+    opt = clone_query->getOption(D6O_CLIENTID);
+    ASSERT_TRUE(opt);
+    EXPECT_TRUE(opt->equals(*orig_clientid));
+
+    // Verify the D6O_ORO option
+    opt = clone_query->getOption(D6O_ORO);
+    ASSERT_TRUE(opt);
+    OptionUint16ArrayPtr clone_oro = boost::dynamic_pointer_cast<OptionUint16Array>(opt);
+    ASSERT_TRUE(clone_oro);
+    EXPECT_TRUE(clone_oro->equals(*orig_oro));
+}
+
+// This test verifies that D6O_CLIENT_DATA options can be created, packed,
+// and unpacked correctly.
+TEST_F(Pkt6Test, clientDataOption) {
+
+    OptionDefinitionPtr def = LibDHCP::getOptionDef(DHCP6_OPTION_SPACE, D6O_CLIENT_DATA);
+    ASSERT_TRUE(def) << "D6O_CLIENT_DATA is not undefined";
+
+    OptionCustomPtr cd_option(new OptionCustom(*def, Option::V6));
+    ASSERT_TRUE(cd_option);
+
+    // Now add supported sub-options: D6O_CLIENTID, D6O_IAADR, D6O_IAAPREFIX,
+    // and D6O_CLTT
+
+    // Add a D6O_CLIENTID option
+    DuidPtr duid(new DUID(DUID::fromText("0102020202030303030303")));
+    OptionPtr orig_clientid(new Option(Option::V6, D6O_CLIENTID, OptionBuffer(
+                                       duid->getDuid().begin(), duid->getDuid().end())));
+    ASSERT_NO_THROW_LOG(cd_option->addOption(orig_clientid));
+
+    // Add a D6O_IAADDR option
+    Option6IAAddrPtr orig_iaaddr1(new Option6IAAddr(D6O_IAADDR, IOAddress("2001:db8::1"), 0, 0));
+    ASSERT_TRUE(orig_iaaddr1);
+    ASSERT_NO_THROW_LOG(cd_option->addOption(orig_iaaddr1));
+
+    // Add another D6O_IAADDR option
+    Option6IAAddrPtr orig_iaaddr2(new Option6IAAddr(D6O_IAADDR, IOAddress("2001:db8::2"), 0, 0));
+    ASSERT_TRUE(orig_iaaddr2);
+    ASSERT_NO_THROW_LOG(cd_option->addOption(orig_iaaddr2));
+
+    // Add a D6O_IAPREFIX option
+    Option6IAAddrPtr orig_iaprefix1(new Option6IAPrefix(D6O_IAPREFIX, IOAddress("2001:db8:1::"), 64, 0, 0));
+    ASSERT_TRUE(orig_iaprefix1);
+    ASSERT_NO_THROW_LOG(cd_option->addOption(orig_iaprefix1));
+
+    // Add another D6O_IAPREFIX option
+    Option6IAAddrPtr orig_iaprefix2(new Option6IAPrefix(D6O_IAPREFIX, IOAddress("2001:db8:2::"), 64, 0, 0));
+    ASSERT_TRUE(orig_iaprefix2);
+    ASSERT_NO_THROW_LOG(cd_option->addOption(orig_iaprefix2));
+
+    // Add a D6O_CLT_TIME option
+    OptionUint32Ptr orig_cltt(new OptionInt<uint32_t>(Option::V6, D6O_CLT_TIME, 4000));
+    ASSERT_TRUE(orig_cltt);
+    ASSERT_NO_THROW_LOG(cd_option->addOption(orig_cltt));
+
+    // Now let's create a packet to which to add our new client data option.
+    Pkt6Ptr orig(new Pkt6(DHCPV6_LEASEQUERY_REPLY, 0x2312));
+    orig->addOption(cd_option);
+    ASSERT_NO_THROW_LOG(orig->pack());
+
+    // Now create second packet,based on assembled data from the first one
+    Pkt6Ptr clone(new Pkt6(static_cast<const uint8_t*>
+                           (orig->getBuffer().getData()),
+                            orig->getBuffer().getLength()));
+
+    // Unpack it.
+    ASSERT_NO_THROW_LOG(clone->unpack());
+
+    // We should be able to find our client data option.
+    OptionPtr opt;
+    opt = clone->getOption(D6O_CLIENT_DATA);
+    ASSERT_TRUE(opt);
+    OptionCustomPtr clone_cd_option = boost::dynamic_pointer_cast<OptionCustom>(opt);
+    ASSERT_TRUE(clone_cd_option);
+
+    // Verify the suboptions.
+    opt = clone_cd_option->getOption(D6O_CLIENTID);
+    ASSERT_TRUE(opt);
+    EXPECT_TRUE(opt->equals(*orig_clientid));
+
+    // Verify the first address option
+    opt = clone_cd_option->getOption(D6O_IAADDR);
+    ASSERT_TRUE(opt);
+    Option6IAAddrPtr clone_iaaddr = boost::dynamic_pointer_cast<Option6IAAddr>(opt);
+    ASSERT_TRUE(clone_iaaddr);
+    EXPECT_TRUE(clone_iaaddr->equals(*orig_iaaddr1));
+
+    // Verify the second address option.
+    opt = clone_cd_option->getOption(D6O_IAADDR);
+    ASSERT_TRUE(opt);
+    clone_iaaddr = boost::dynamic_pointer_cast<Option6IAAddr>(opt);
+    ASSERT_TRUE(clone_iaaddr);
+    EXPECT_TRUE(clone_iaaddr->equals(*orig_iaaddr2));
+
+    // Verify the first prefix option.
+    opt = clone_cd_option->getOption(D6O_IAPREFIX);
+    ASSERT_TRUE(opt);
+    Option6IAPrefixPtr clone_iaprefix = boost::dynamic_pointer_cast<Option6IAPrefix>(opt);
+    ASSERT_TRUE(clone_iaprefix);
+    EXPECT_TRUE(clone_iaprefix->equals(*orig_iaprefix1));
+
+    // Verify the second prefix option.
+    opt = clone_cd_option->getOption(D6O_IAPREFIX);
+    ASSERT_TRUE(opt);
+    clone_iaprefix = boost::dynamic_pointer_cast<Option6IAPrefix>(opt);
+    ASSERT_TRUE(clone_iaprefix);
+    EXPECT_TRUE(clone_iaprefix->equals(*orig_iaprefix2));
+
+    // Verify the CLT option.
+    opt = clone_cd_option->getOption(D6O_CLT_TIME);
+    ASSERT_TRUE(opt);
+    OptionUint32Ptr clone_cltt = boost::dynamic_pointer_cast<OptionUint32>(opt);
+    ASSERT_TRUE(clone_cltt);
+    EXPECT_TRUE(clone_cltt->equals(*orig_cltt));
+}
+
+// This test verifies that D6O_LQ_RELAY_DATA options can be created, packed,
+// and unpacked correctly.
+TEST_F(Pkt6Test, relayDataOption) {
+    OptionDefinitionPtr def = LibDHCP::getOptionDef(DHCP6_OPTION_SPACE, D6O_LQ_RELAY_DATA);
+    ASSERT_TRUE(def) << "D6O_LQ_RELAY_DATA is not undefined";
+
+    OptionCustomPtr rd_option(new OptionCustom(*def, Option::V6));
+    ASSERT_TRUE(rd_option);
+
+    // Write out the peer address.
+    IOAddress orig_address("2001:db8::1");
+    rd_option->writeAddress(orig_address, 0);
+
+    // Write out the binary data (in real life this is a RELAY_FORW message)
+    std::vector<uint8_t>orig_data({ 01,02,03,04,05,06 });
+    rd_option->writeBinary(orig_data, 1);
+
+    // Now let's create a packet to which to add our new relay data option.
+    Pkt6Ptr orig(new Pkt6(DHCPV6_LEASEQUERY_REPLY, 0x2312));
+    orig->addOption(rd_option);
+    ASSERT_NO_THROW_LOG(orig->pack());
+
+    // Now create second packet,based on assembled data from the first one
+    Pkt6Ptr clone(new Pkt6(static_cast<const uint8_t*>
+                           (orig->getBuffer().getData()),
+                            orig->getBuffer().getLength()));
+    // Unpack it.
+    ASSERT_NO_THROW_LOG(clone->unpack());
+
+    // We should be able to find our client data option.
+    OptionPtr opt;
+    opt = clone->getOption(D6O_LQ_RELAY_DATA);
+    ASSERT_TRUE(opt);
+    OptionCustomPtr clone_rd_option = boost::dynamic_pointer_cast<OptionCustom>(opt);
+    ASSERT_TRUE(clone_rd_option);
+
+    // Verify the address field.
+    IOAddress clone_addr("::");
+    ASSERT_NO_THROW_LOG(clone_addr = clone_rd_option->readAddress(0));
+    EXPECT_EQ(orig_address, clone_addr);
+
+    // Verify the binary field
+    OptionBuffer clone_data;
+    ASSERT_NO_THROW_LOG(clone_data = clone_rd_option->readBinary(1));
+    EXPECT_EQ(orig_data, clone_data);
+}
+
 }
