@@ -1,4 +1,4 @@
-// Copyright (C) 2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2019-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -209,7 +209,7 @@ CBControlDHCPv4::databaseConfigApply(const BackendSelector& backend_selector,
     LOG_INFO(dhcpsrv_logger, DHCPSRV_CFGMGR_CONFIG4_MERGED);
 
     if (!audit_entries.empty() &&
-        HooksManager::getHooksManager().calloutsPresent(hooks_.hook_index_cb4_updated_)) {
+        HooksManager::calloutsPresent(hooks_.hook_index_cb4_updated_)) {
         CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
 
         // Use the RAII wrapper to make sure that the callout handle state is
