@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -276,6 +276,10 @@ Network::toElement() const {
         map->set("hostname-char-replacement", Element::create(hostname_char_replacement_));
     }
 
+    if (!store_extended_info_.unspecified()) {
+        map->set("store-extended-info", Element::create(store_extended_info_));
+    }
+
     return (map);
 }
 
