@@ -16,6 +16,7 @@
 #include <log/logger_manager.h>
 #include <log/logger_specification.h>
 #include <dhcp/pkt.h> // Needed for HWADDR_SOURCE_*
+#include <stats/stats_mgr.h>
 #include <util/strutil.h>
 
 #include <list>
@@ -41,8 +42,6 @@ SrvConfig::SrvConfig()
       cfg_host_operations6_(CfgHostOperations::createConfig6()),
       class_dictionary_(new ClientClassDictionary()),
       decline_timer_(0), echo_v4_client_id_(true), dhcp4o6_port_(0),
-      server_threads_(0),
-      server_max_thread_queue_size_(0),
       d2_client_config_(new D2ClientConfig()),
       configured_globals_(Element::createMap()),
       cfg_consist_(new CfgConsistency()) {
@@ -61,8 +60,6 @@ SrvConfig::SrvConfig(const uint32_t sequence)
       cfg_host_operations6_(CfgHostOperations::createConfig6()),
       class_dictionary_(new ClientClassDictionary()),
       decline_timer_(0), echo_v4_client_id_(true), dhcp4o6_port_(0),
-      server_threads_(0),
-      server_max_thread_queue_size_(0),
       d2_client_config_(new D2ClientConfig()),
       configured_globals_(Element::createMap()),
       cfg_consist_(new CfgConsistency()) {
@@ -254,7 +251,6 @@ SrvConfig::mergeGlobals(SrvConfig& other) {
     }
 }
 
-
 void
 SrvConfig::removeStatistics() {
     // Removes statistics for v4 and v6 subnets
@@ -265,6 +261,35 @@ SrvConfig::removeStatistics() {
 
 void
 SrvConfig::updateStatistics() {
+    // Update default sample limits.
+    stats::StatsMgr& stats_mgr = stats::StatsMgr::instance();
+    ConstElementPtr samples =
+        getConfiguredGlobal("statistic-default-sample-count");
+    uint32_t max_samples = 0;
+    if (samples) {
+        max_samples = samples->intValue();
+        stats_mgr.setMaxSampleCountDefault(max_samples);
+        if (max_samples != 0) {
+            stats_mgr.setMaxSampleCountAll(max_samples);
+        }
+    }
+    ConstElementPtr duration =
+        getConfiguredGlobal("statistic-default-sample-age");
+    if (duration) {
+        int64_t time_duration = duration->intValue();
+        int64_t hours = time_duration / 3600;
+        time_duration -= hours * 3600;
+        int64_t minutes = time_duration / 60;
+        time_duration -= minutes * 60;
+        int64_t seconds = time_duration;
+        auto max_age =
+            boost::posix_time::time_duration(hours, minutes, seconds, 0);
+        stats_mgr.setMaxSampleAgeDefault(max_age);
+        if (max_samples == 0) {
+            stats_mgr.setMaxSampleAgeAll(max_age);
+        }
+    }
+
     // Updating subnet statistics involves updating lease statistics, which
     // is done by the LeaseMgr.  Since servers with subnets, must have a
     // LeaseMgr, we do not bother updating subnet stats for servers without
@@ -414,7 +439,6 @@ SrvConfig::toElement() const {
     // Set dhcp4o6-port
     dhcp->set("dhcp4o6-port",
               Element::create(static_cast<int>(dhcp4o6_port_)));
-
     // Set dhcp-ddns
     dhcp->set("dhcp-ddns", d2_client_config_->toElement());
     // Set interfaces-config
@@ -591,12 +615,18 @@ SrvConfig::toElement() const {
         dhcp->set("dhcp-queue-control", dhcp_queue_control);
     }
 
+    // Set multi-threading (if it exists)
+    data::ConstElementPtr dhcp_multi_threading = getDHCPMultiThreading();
+    if (dhcp_multi_threading) {
+        dhcp->set("multi-threading", dhcp_multi_threading);
+    }
+
     return (result);
 }
 
 DdnsParamsPtr
 SrvConfig::getDdnsParams(const Subnet4Ptr& subnet) const {
-    return (DdnsParamsPtr(new DdnsParams(subnet, 
+    return (DdnsParamsPtr(new DdnsParams(subnet,
                                          getD2ClientConfig()->getEnableUpdates())));
 }
 
