@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,36 +9,41 @@
 
 #include <asiolink/io_address.h>
 #include <cc/data.h>
+#include <cc/user_context.h>
 #include <dhcp/option_space_container.h>
-#include <dhcpsrv/assignable_network.h>
 #include <dhcpsrv/lease.h>
+#include <dhcpsrv/network.h>
 #include <dhcpsrv/pool.h>
 #include <dhcpsrv/subnet_id.h>
 #include <dhcpsrv/triplet.h>
-
 #include <boost/multi_index/mem_fun.hpp>
 #include <boost/multi_index/indexed_by.hpp>
 #include <boost/multi_index/ordered_index.hpp>
 #include <boost/multi_index/random_access_index.hpp>
 #include <boost/multi_index_container.hpp>
+#include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/pointer_cast.hpp>
+#include <boost/scoped_ptr.hpp>
 #include <boost/shared_ptr.hpp>
+#include <cstdint>
+#include <map>
+#include <mutex>
+#include <utility>
 
 namespace isc {
 namespace dhcp {
 
-class Subnet : public data::CfgToElement {
-
-    // Assignable network is our friend to allow it to call
-    // @ref Subnet::setSharedNetwork private function.
-    friend class AssignableNetwork;
-
+class Subnet : public virtual Network {
 public:
 
-    /// @brief checks if specified address is in range
+    /// @brief checks if specified address is in range.
+    ///
+    /// @param addr this address will be checked if it is included in a specific
+    ///        range
+    /// @return true if address is in range, false otherwise
     bool inRange(const isc::asiolink::IOAddress& addr) const;
 
-    /// @brief checks if the specified address is in pools
+    /// @brief checks if the specified address is in pools.
     ///
     /// Note the difference between inRange() and inPool() for addresses
     /// (i.e. *not* prefixes). For a given subnet (e.g. 2001::/64) there
@@ -54,37 +59,69 @@ public:
     /// @return true if the address is in any of the pools
     bool inPool(Lease::Type type, const isc::asiolink::IOAddress& addr) const;
 
-    /// @brief returns the last address that was tried from this pool
+    /// @brief checks if the specified address is in allowed pools.
+    ///
+    /// This takes also into account client classes
+    ///
+    /// @param type type of pools to iterate over
+    /// @param addr this address will be checked if it belongs to any pools in
+    ///        that subnet
+    /// @param client_classes client class list which must be allowed
+    /// @return true if the address is in any of the allowed pools
+    bool inPool(Lease::Type type,
+                const isc::asiolink::IOAddress& addr,
+                const ClientClasses& client_classes) const;
+
+    /// @brief returns the last address that was tried from this subnet.
     ///
     /// This method returns the last address that was attempted to be allocated
     /// from this subnet. This is used as helper information for the next
     /// iteration of the allocation algorithm.
     ///
-    /// @todo: Define map<SubnetID, IOAddress> somewhere in the
+    /// @note: this routine is Kea thread safe.
+    ///
+    /// @todo: Define map<SubnetID, ClientClass, IOAddress> somewhere in the
     ///        AllocEngine::IterativeAllocator and keep the data there
     ///
     /// @param type lease type to be returned
-    /// @return address/prefix that was last tried from this pool
+    /// @return address/prefix that was last tried from this subnet
     isc::asiolink::IOAddress getLastAllocated(Lease::Type type) const;
 
-    /// @brief sets the last address that was tried from this pool
+    /// @brief Returns the timestamp when the @c setLastAllocated function
+    /// was called.
+    ///
+    /// @note: this routine is Kea thread safe.
+    ///
+    /// @param lease_type Lease type for which last allocation timestamp should
+    /// be returned.
+    ///
+    /// @return Time when a lease of a specified type has been allocated from
+    /// this subnet. The negative infinity time is returned if a lease type is
+    /// not recognized (which is unlikely).
+    boost::posix_time::ptime
+    getLastAllocatedTime(const Lease::Type& lease_type) const;
+
+    /// @brief sets the last address that was tried from this subnet.
     ///
     /// This method sets the last address that was attempted to be allocated
     /// from this subnet. This is used as helper information for the next
     /// iteration of the allocation algorithm.
     ///
-    /// @todo: Define map<SubnetID, IOAddress> somewhere in the
+    /// @note: this routine is Kea thread safe.
+    ///
+    /// @todo: Define map<SubnetID, ClientClass, IOAddress> somewhere in the
     ///        AllocEngine::IterativeAllocator and keep the data there
     /// @param addr address/prefix to that was tried last
     /// @param type lease type to be set
     void setLastAllocated(Lease::Type type,
                           const isc::asiolink::IOAddress& addr);
 
-    /// @brief Returns unique ID for that subnet
+    /// @brief Returns unique ID for that subnet.
+    ///
     /// @return unique ID for that subnet
     SubnetID getID() const { return (id_); }
 
-    /// @brief Returns subnet parameters (prefix and prefix length)
+    /// @brief Returns subnet parameters (prefix and prefix length).
     ///
     /// @return (prefix, prefix length) pair
     std::pair<isc::asiolink::IOAddress, uint8_t> get() const {
@@ -115,13 +152,14 @@ public:
     /// within the subnet.
     void addPool(const PoolPtr& pool);
 
-    /// @brief Deletes all pools of specified type
+    /// @brief Deletes all pools of specified type.
     ///
     /// This method is used for testing purposes only
+    ///
     /// @param type type of pools to be deleted
     void delPools(Lease::Type type);
 
-    /// @brief Returns a pool that specified address belongs to
+    /// @brief Returns a pool that specified address belongs to.
     ///
     /// This method uses binary search to retrieve the pool. Thus, the number
     /// of comparisons performed by this method is logarithmic in the number
@@ -142,7 +180,18 @@ public:
     const PoolPtr getPool(Lease::Type type, const isc::asiolink::IOAddress& addr,
                           bool anypool = true) const;
 
-    /// @brief Returns a pool without any address specified
+    /// @brief Returns a pool that specified address belongs to with classes.
+    ///
+    /// Variant using only pools allowing given classes.
+    ///
+    /// @param type pool type that the pool is looked for
+    /// @param client_classes client class list which must be allowed
+    /// @param addr address that the returned pool should cover (optional)
+    const PoolPtr getPool(Lease::Type type,
+                          const ClientClasses& client_classes,
+                          const isc::asiolink::IOAddress& addr) const;
+
+    /// @brief Returns a pool without any address specified.
     ///
     /// @param type pool type that the pool is looked for
     /// @return returns one of the pools defined
@@ -150,13 +199,13 @@ public:
         return (getPool(type, default_pool()));
     }
 
-    /// @brief Returns the default address that will be used for pool selection
+    /// @brief Returns the default address that will be used for pool selection.
     ///
     /// It must be implemented in derived classes (should return :: for Subnet6
-    /// and 0.0.0.0 for Subnet4)
+    /// and 0.0.0.0 for Subnet4).
     virtual isc::asiolink::IOAddress default_pool() const = 0;
 
-    /// @brief Returns all pools (const variant)
+    /// @brief Returns all pools (const variant).
     ///
     /// The reference is only valid as long as the object that returned it.
     ///
@@ -164,18 +213,27 @@ public:
     /// @return a collection of all pools
     const PoolCollection& getPools(Lease::Type type) const;
 
-    /// @brief Returns the number of possible leases for specified lease type
+    /// @brief Returns the number of possible leases for specified lease type.
     ///
     /// @param type type of the lease
     uint64_t getPoolCapacity(Lease::Type type) const;
 
+    /// @brief Returns the number of possible leases for specified lease type
+    /// allowed for a client which belongs to classes.
+    ///
+    /// @param type type of the lease
+    /// @param client_classes list of classes the client belongs to
+    /// @return number of leases matching lease type and classes
+    uint64_t getPoolCapacity(Lease::Type type,
+                             const ClientClasses& client_classes) const;
+
     /// @brief Returns textual representation of the subnet (e.g.
-    /// "2001:db8::/64")
+    /// "2001:db8::/64").
     ///
     /// @return textual representation
     virtual std::string toText() const;
 
-    /// @brief Resets subnet-id counter to its initial value (1)
+    /// @brief Resets subnet-id counter to its initial value (1).
     ///
     /// This should be called during reconfiguration, before any new
     /// subnet objects are created. It will ensure that the subnet_id will
@@ -203,11 +261,9 @@ public:
     template<typename SharedNetworkPtrType>
     void getSharedNetwork(SharedNetworkPtrType& shared_network) const {
         shared_network = boost::dynamic_pointer_cast<
-            typename SharedNetworkPtrType::element_type>(shared_network_.lock());
+            typename SharedNetworkPtrType::element_type>(parent_network_.lock());
     }
 
-private:
-
     /// @brief Assigns shared network to a subnet.
     ///
     /// This method replaces any shared network associated with a subnet with
@@ -216,24 +272,33 @@ private:
     /// @param shared_network Pointer to a new shared network to be associated
     /// with the subnet.
     void setSharedNetwork(const NetworkPtr& shared_network) {
-        shared_network_ = shared_network;
+        parent_network_ = shared_network;
     }
 
-public:
-
-    /// @brief Sets user context.
-    /// @param ctx user context to be stored.
-    void setContext(const data::ConstElementPtr& ctx) {
-        user_context_ = ctx;
+    /// @brief Returns shared network name.
+    ///
+    /// @return shared network name
+    std::string getSharedNetworkName() const {
+        return (shared_network_name_);
     }
 
-    /// @brief Returns const pointer to the user context.
-    data::ConstElementPtr getContext() const {
-        return (user_context_);
+    /// @brief Sets new shared network name.
+    ///
+    /// In certain cases the subnet must be associated with the shared network
+    /// but the shared network object is not available. In particular, subnets
+    /// are returned from the configuration database with only names of the
+    /// shared networks. The actual shared networks must be fetched from the
+    /// database using a separate query. In order to not loose associations
+    /// of subnets with shared networks, the configuration backends will use
+    /// this method to store the shared network names. The servers will later
+    /// use those names to associate subnets with shared network instances.
+    ///
+    /// @param shared_network_name New shared network name.
+    void setSharedNetworkName(const std::string& shared_network_name) {
+        shared_network_name_ = shared_network_name;
     }
 
-protected:
-    /// @brief Returns all pools (non-const variant)
+    /// @brief Returns all pools (non-const variant).
     ///
     /// The reference is only valid as long as the object that returned it.
     ///
@@ -241,7 +306,9 @@ protected:
     /// @return a collection of all pools
     PoolCollection& getPoolsWritable(Lease::Type type);
 
-    /// @brief Protected constructor
+protected:
+
+    /// @brief Protected constructor.
     //
     /// By making the constructor protected, we make sure that no one will
     /// ever instantiate that class. Subnet4 and Subnet6 should be used instead.
@@ -258,13 +325,13 @@ protected:
     Subnet(const isc::asiolink::IOAddress& prefix, uint8_t len,
            const SubnetID id);
 
-    /// @brief virtual destructor
+    /// @brief virtual destructor.
     ///
     /// A virtual destructor is needed because other classes
     /// derive from this class.
     virtual ~Subnet() { };
 
-    /// @brief keeps the subnet-id value
+    /// @brief keeps the subnet-id value.
     ///
     /// It is incremented every time a new Subnet object is created. It is reset
     /// (@ref resetSubnetID) every time reconfiguration
@@ -273,7 +340,7 @@ protected:
     /// Static value initialized in subnet.cc.
     static SubnetID static_id_;
 
-    /// @brief returns the next unique Subnet-ID
+    /// @brief returns the next unique Subnet-ID.
     ///
     /// This method generates and returns the next unique subnet-id.
     /// It is a strictly monotonously increasing value (1,2,3,...) for
@@ -282,10 +349,14 @@ protected:
     ///
     /// @return the next unique Subnet-ID
     static SubnetID generateNextID() {
+        if (static_id_ == SUBNET_ID_MAX) {
+            resetSubnetID();
+        }
+
         return (static_id_++);
     }
 
-    /// @brief Checks if used pool type is valid
+    /// @brief Checks if used pool type is valid.
     ///
     /// Allowed type for Subnet4 is Pool::TYPE_V4.
     /// Allowed types for Subnet6 are Pool::TYPE_{IA,TA,PD}.
@@ -295,11 +366,20 @@ protected:
     /// @throw BadValue if invalid value is used
     virtual void checkType(Lease::Type type) const = 0;
 
-    /// @brief returns a sum of possible leases in all pools
+    /// @brief Returns a sum of possible leases in all pools.
+    ///
     /// @param pools list of pools
     /// @return sum of possible leases
     uint64_t sumPoolCapacity(const PoolCollection& pools) const;
 
+    /// @brief Returns a sum of possible leases in all pools allowing classes.
+    ///
+    /// @param pools list of pools
+    /// @param client_classes list of classes
+    /// @return sum of possible/allowed leases
+    uint64_t sumPoolCapacity(const PoolCollection& pools,
+                             const ClientClasses& client_classes) const;
+
     /// @brief Checks if the specified pool overlaps with an existing pool.
     ///
     /// @param pool_type Pool type.
@@ -307,7 +387,7 @@ protected:
     /// it overlaps with any existing pool within this subnet.
     ///
     /// @return true if pool overlaps with an existing pool of a specified
-    /// type.
+    /// type, false otherwise
     bool poolOverlaps(const Lease::Type& pool_type, const PoolPtr& pool) const;
 
     /// @brief Unparse a subnet object.
@@ -315,28 +395,39 @@ protected:
     /// @return A pointer to unparsed subnet configuration.
     virtual data::ElementPtr toElement() const;
 
+    /// @brief Converts subnet prefix to a pair of prefix/length pair.
+    ///
+    /// IPv4 and IPv6 specific conversion functions should apply extra checks
+    /// on the returned values, i.e. whether length is in range and the IP
+    /// address has a valid type.
+    ///
+    /// @param prefix Prefix to be parsed.
+    /// @throw BadValue if provided prefix is not valid.
+    static std::pair<asiolink::IOAddress, uint8_t>
+    parsePrefixCommon(const std::string& prefix);
+
     /// @brief subnet-id
     ///
     /// Subnet-id is a unique value that can be used to find or identify
     /// a Subnet4 or Subnet6.
     SubnetID id_;
 
-    /// @brief collection of IPv4 or non-temporary IPv6 pools in that subnet
+    /// @brief collection of IPv4 or non-temporary IPv6 pools in that subnet.
     PoolCollection pools_;
 
-    /// @brief collection of IPv6 temporary address pools in that subnet
+    /// @brief collection of IPv6 temporary address pools in that subnet.
     PoolCollection pools_ta_;
 
-    /// @brief collection of IPv6 prefix pools in that subnet
+    /// @brief collection of IPv6 prefix pools in that subnet.
     PoolCollection pools_pd_;
 
-    /// @brief a prefix of the subnet
+    /// @brief a prefix of the subnet.
     isc::asiolink::IOAddress prefix_;
 
-    /// @brief a prefix length of the subnet
+    /// @brief a prefix length of the subnet.
     uint8_t prefix_len_;
 
-    /// @brief last allocated address
+    /// @brief last allocated address.
     ///
     /// This is the last allocated address that was previously allocated from
     /// this particular subnet. Some allocation algorithms (e.g. iterative) use
@@ -347,24 +438,78 @@ protected:
     /// fully trusted.
     isc::asiolink::IOAddress last_allocated_ia_;
 
-    /// @brief last allocated temporary address
+    /// @brief last allocated temporary address.
     ///
     /// See @ref last_allocated_ia_ for details.
     isc::asiolink::IOAddress last_allocated_ta_;
 
-    /// @brief last allocated IPv6 prefix
+    /// @brief last allocated IPv6 prefix.
     ///
     /// See @ref last_allocated_ia_ for details.
     isc::asiolink::IOAddress last_allocated_pd_;
 
-    /// @brief Name of the network interface (if connected directly)
+    /// @brief Timestamp indicating when a lease of a specified type has been
+    /// last allocated from this subnet.
+    ///
+    /// @note: This map is protected by the mutex.
+    std::map<Lease::Type, boost::posix_time::ptime> last_allocated_time_;
+
+    /// @brief Name of the network interface (if connected directly).
     std::string iface_;
 
-    /// @brief Pointer to a shared network that subnet belongs to.
-    WeakNetworkPtr shared_network_;
+    /// @brief Shared network name.
+    std::string shared_network_name_;
 
-    /// @brief Pointer to the user context (may be NULL)
-    data::ConstElementPtr user_context_;
+private:
+
+    /// @brief returns the last address that was tried from this subnet.
+    ///
+    /// Should be called in a thread safe context.
+    ///
+    /// This method returns the last address that was attempted to be allocated
+    /// from this subnet. This is used as helper information for the next
+    /// iteration of the allocation algorithm.
+    ///
+    /// @todo: Define map<SubnetID, ClientClass, IOAddress> somewhere in the
+    ///        AllocEngine::IterativeAllocator and keep the data there
+    ///
+    /// @param type lease type to be returned
+    /// @return address/prefix that was last tried from this subnet
+    isc::asiolink::IOAddress getLastAllocatedInternal(Lease::Type type) const;
+
+    /// @brief Returns the timestamp when the @c setLastAllocated function
+    /// was called.
+    ///
+    /// Should be called in a thread safe context.
+    ///
+    /// @param lease_type Lease type for which last allocation timestamp should
+    /// be returned.
+    ///
+    /// @return Time when a lease of a specified type has been allocated from
+    /// this subnet. The negative infinity time is returned if a lease type is
+    /// not recognized (which is unlikely).
+    boost::posix_time::ptime
+    getLastAllocatedTimeInternal(const Lease::Type& lease_type) const;
+
+    /// @brief sets the last address that was tried from this subnet.
+    ///
+    /// Should be called in a thread safe context.
+    ///
+    /// This method sets the last address that was attempted to be allocated
+    /// from this subnet. This is used as helper information for the next
+    /// iteration of the allocation algorithm.
+    ///
+    /// @note: this routine is Kea thread safe.
+    ///
+    /// @todo: Define map<SubnetID, ClientClass, IOAddress> somewhere in the
+    ///        AllocEngine::IterativeAllocator and keep the data there
+    /// @param addr address/prefix to that was tried last
+    /// @param type lease type to be set
+    void setLastAllocatedInternal(Lease::Type type,
+                                  const isc::asiolink::IOAddress& addr);
+
+    /// @brief Mutex to protect the internal state.
+    boost::scoped_ptr<std::mutex> mutex_;
 };
 
 /// @brief A generic pointer to either Subnet4 or Subnet6 object
@@ -382,10 +527,12 @@ typedef boost::shared_ptr<Subnet4> Subnet4Ptr;
 /// @brief A configuration holder for IPv4 subnet.
 ///
 /// This class represents an IPv4 subnet.
+/// @note Subnet and Network use virtual inheritance to avoid
+/// a diamond issue with UserContext
 class Subnet4 : public Subnet, public Network4 {
 public:
 
-    /// @brief Constructor with all parameters
+    /// @brief Constructor with all parameters.
     ///
     /// This constructor calls Subnet::Subnet, where subnet-id is generated.
     ///
@@ -402,6 +549,29 @@ public:
             const Triplet<uint32_t>& valid_lifetime,
             const SubnetID id = 0);
 
+    /// @brief Factory function creating an instance of the @c Subnet4.
+    ///
+    /// This function should be used to create an instance of the subnet
+    /// object within a hooks library in cases when the library may be
+    /// unloaded before the object is destroyed. This ensures that the
+    /// ownership of the object by the Kea process is retained.
+    ///
+    /// @param prefix Subnet4 prefix
+    /// @param length prefix length
+    /// @param t1 renewal timer (in seconds)
+    /// @param t2 rebind timer (in seconds)
+    /// @param valid_lifetime preferred lifetime of leases (in seconds)
+    /// @param id arbitrary subnet id, default value of 0 triggers
+    /// autogeneration of subnet id
+    ///
+    /// @return Pointer to the @c Subnet4 instance.
+    static Subnet4Ptr
+    create(const isc::asiolink::IOAddress& prefix, uint8_t length,
+           const Triplet<uint32_t>& t1,
+           const Triplet<uint32_t>& t2,
+           const Triplet<uint32_t>& valid_lifetime,
+           const SubnetID id = 0);
+
     /// @brief Returns next subnet within shared network.
     ///
     /// If the current subnet doesn't belong to any shared network or if
@@ -445,38 +615,6 @@ public:
     virtual bool
     clientSupported(const isc::dhcp::ClientClasses& client_classes) const;
 
-    /// @brief Sets siaddr for the Subnet4
-    ///
-    /// Will be used for siaddr field (the next server) that typically is used
-    /// as TFTP server. If not specified, the default value of 0.0.0.0 is
-    /// used.
-    void setSiaddr(const isc::asiolink::IOAddress& siaddr);
-
-    /// @brief Returns siaddr for this subnet
-    ///
-    /// @return siaddr value
-    isc::asiolink::IOAddress getSiaddr() const;
-
-    /// @brief Sets server hostname for the Subnet4 
-    ///
-    /// Will be used for server hostname field (may be empty if not defined)
-    void setSname(const std::string& sname);
-
-    /// @brief Returns server hostname for this subnet
-    ///
-    /// @return server hostname value
-    const std::string& getSname() const;
-
-    /// @brief Sets boot file name for the Subnet4 
-    ///
-    /// Will be used for boot file name (may be empty if not defined)
-    void setFilename(const std::string& filename);
-
-    /// @brief Returns boot file name for this subnet
-    ///
-    /// @return boot file name value
-    const std::string& getFilename() const;
-
     /// @brief Returns DHCP4o6 configuration parameters.
     ///
     /// This structure is always available. If the 4o6 is not enabled, its
@@ -498,15 +636,23 @@ public:
     /// @return A pointer to unparsed subnet configuration.
     virtual data::ElementPtr toElement() const;
 
+    /// @brief Converts subnet prefix to a pair of prefix/length pair.
+    ///
+    /// @param prefix Prefix to be parsed.
+    /// @throw BadValue if provided invalid IPv4 prefix.
+    static std::pair<asiolink::IOAddress, uint8_t>
+    parsePrefix(const std::string& prefix);
+
 private:
 
-    /// @brief Returns default address for pool selection
+    /// @brief Returns default address for pool selection.
+    ///
     /// @return ANY IPv4 address
     virtual isc::asiolink::IOAddress default_pool() const {
         return (isc::asiolink::IOAddress("0.0.0.0"));
     }
 
-    /// @brief Checks if used pool type is valid
+    /// @brief Checks if used pool type is valid.
     ///
     /// Allowed type for Subnet4 is Pool::TYPE_V4.
     ///
@@ -514,15 +660,6 @@ private:
     /// @throw BadValue if invalid value is used
     virtual void checkType(Lease::Type type) const;
 
-    /// @brief siaddr value for this subnet
-    isc::asiolink::IOAddress siaddr_;
-
-    /// @brief server hostname for this subnet
-    std::string sname_;
-
-    /// @brief boot file name for this subnet
-    std::string filename_;
-
     /// @brief All the information related to DHCP4o6
     Cfg4o6 dhcp4o6_;
 };
@@ -538,10 +675,12 @@ typedef boost::shared_ptr<Subnet6> Subnet6Ptr;
 /// @brief A configuration holder for IPv6 subnet.
 ///
 /// This class represents an IPv6 subnet.
+/// @note Subnet and Network use virtual inheritance to avoid
+/// a diamond issue with UserContext
 class Subnet6 : public Subnet, public Network6 {
 public:
 
-    /// @brief Constructor with all parameters
+    /// @brief Constructor with all parameters.
     ///
     /// This constructor calls Subnet::Subnet, where subnet-id is generated.
     ///
@@ -560,6 +699,31 @@ public:
             const Triplet<uint32_t>& valid_lifetime,
             const SubnetID id = 0);
 
+    /// @brief Factory function creating an instance of the @c Subnet4.
+    ///
+    /// This function should be used to create an instance of the subnet
+    /// object within a hooks library in cases when the library may be
+    /// unloaded before the object is destroyed. This ensures that the
+    /// ownership of the object by the Kea process is retained.
+    ///
+    /// @param prefix Subnet6 prefix
+    /// @param length prefix length
+    /// @param t1 renewal timer (in seconds)
+    /// @param t2 rebind timer (in seconds)
+    /// @param preferred_lifetime preferred lifetime of leases (in seconds)
+    /// @param valid_lifetime preferred lifetime of leases (in seconds)
+    /// @param id arbitrary subnet id, default value of 0 triggers
+    /// autogeneration of subnet id
+    ///
+    /// @return Pointer to the @c Subnet6 instance.
+    static Subnet6Ptr
+    create(const isc::asiolink::IOAddress& prefix, uint8_t length,
+           const Triplet<uint32_t>& t1,
+           const Triplet<uint32_t>& t2,
+           const Triplet<uint32_t>& preferred_lifetime,
+           const Triplet<uint32_t>& valid_lifetime,
+           const SubnetID id = 0);
+
     /// @brief Returns next subnet within shared network.
     ///
     /// If the current subnet doesn't belong to any shared network or if
@@ -608,6 +772,13 @@ public:
     /// @return A pointer to unparsed subnet configuration.
     virtual data::ElementPtr toElement() const;
 
+    /// @brief Converts subnet prefix to a pair of prefix/length pair.
+    ///
+    /// @param prefix Prefix to be parsed.
+    /// @throw BadValue if provided invalid IPv4 prefix.
+    static std::pair<asiolink::IOAddress, uint8_t>
+    parsePrefix(const std::string& prefix);
+
 private:
 
     /// @brief Returns default address for pool selection
@@ -642,7 +813,10 @@ struct SubnetPrefixIndexTag { };
 /// @brief Tag for the index for searching by server identifier.
 struct SubnetServerIdIndexTag { };
 
-/// @brief A collection of @c Subnet4 objects
+/// @brief Tag for the index for searching by subnet modification time.
+struct SubnetModificationTimeIndexTag { };
+
+/// @brief A collection of @c Subnet4 objects.
 ///
 /// This container provides a set of indexes which can be used to retrieve
 /// subnets by various properties.
@@ -686,11 +860,19 @@ typedef boost::multi_index_container<
             boost::multi_index::const_mem_fun<Subnet, std::string, &Subnet::toText>
         >,
 
-        // Fourth index allows for searching using an output from getServerId
+        // Fourth index allows for searching using an output from getServerId.
         boost::multi_index::ordered_non_unique<
             boost::multi_index::tag<SubnetServerIdIndexTag>,
             boost::multi_index::const_mem_fun<Network4, asiolink::IOAddress,
                                               &Network4::getServerId>
+        >,
+
+        // Fifth index allows for searching using subnet modification time.
+        boost::multi_index::ordered_non_unique<
+            boost::multi_index::tag<SubnetModificationTimeIndexTag>,
+            boost::multi_index::const_mem_fun<data::BaseStampedElement,
+                                              boost::posix_time::ptime,
+                                              &data::BaseStampedElement::getModificationTime>
         >
     >
 > Subnet4Collection;
@@ -736,10 +918,53 @@ typedef boost::multi_index_container<
         boost::multi_index::ordered_unique<
             boost::multi_index::tag<SubnetPrefixIndexTag>,
             boost::multi_index::const_mem_fun<Subnet, std::string, &Subnet::toText>
+        >,
+        // Fourth index allows for searching using subnet modification time.
+        boost::multi_index::ordered_non_unique<
+            boost::multi_index::tag<SubnetModificationTimeIndexTag>,
+            boost::multi_index::const_mem_fun<data::BaseStampedElement,
+                                              boost::posix_time::ptime,
+                                              &data::BaseStampedElement::getModificationTime>
         >
     >
 > Subnet6Collection;
 
+/// @brief A class containing static convenience methods to fetch the subnets
+/// from the containers.
+///
+/// @tparam ReturnPtrType Type of the returned object, i.e. @c Subnet4Ptr
+/// or @c Subnet6Ptr.
+/// @tparam CollectionType One of the @c Subnet4Collection or @c Subnet6Collection.
+template<typename ReturnPtrType, typename CollectionType>
+class SubnetFetcher {
+public:
+
+    /// @brief Fetches subnets by id.
+    ///
+    /// @param collection Const reference to the collection from which the
+    /// subnet is to be fetched.
+    /// @param subnet_id Id of the subnet to be fetched.
+    /// @return Pointer to the fetched subnet or null if no such subnet
+    /// could be found.
+    static ReturnPtrType get(const CollectionType& collection,
+                             const SubnetID& subnet_id) {
+        auto& index = collection.template get<SubnetSubnetIdIndexTag>();
+        auto s = index.find(subnet_id);
+        if (s != index.end()) {
+            return (*s);
+        }
+        // No subnet found. Return null pointer.
+        return (ReturnPtrType());
+    }
+};
+
+/// @brief Type of the @c SubnetFetcher used for IPv4.
+using SubnetFetcher4 = SubnetFetcher<Subnet4Ptr, Subnet4Collection>;
+
+/// @brief Type of the @c SubnetFetcher used for IPv6.
+using SubnetFetcher6 = SubnetFetcher<Subnet6Ptr, Subnet6Collection>;
+
+
 //@}
 
 } // end of isc::dhcp namespace
