@@ -1057,7 +1057,6 @@ TEST(CfgSubnets4Test, unparseSubnet) {
     // Unparse
     std::string expected = "[\n"
         "{\n"
-        "    \"comment\": \"foo\",\n"
         "    \"id\": 123,\n"
         "    \"subnet\": \"192.0.2.0/26\",\n"
         "    \"t1-percent\": 0.45,"
@@ -1071,7 +1070,8 @@ TEST(CfgSubnets4Test, unparseSubnet) {
         "    \"4o6-interface-id\": \"\",\n"
         "    \"4o6-subnet\": \"\",\n"
         "    \"option-data\": [ ],\n"
-        "    \"pools\": [ ]\n"
+        "    \"pools\": [ ],\n"
+        "    \"user-context\": { \"comment\": \"foo\" }\n"
         "},{\n"
         "    \"id\": 124,\n"
         "    \"subnet\": \"192.0.2.64/26\",\n"
@@ -1161,10 +1161,10 @@ TEST(CfgSubnets4Test, unparsePool) {
         "    \"option-data\": [],\n"
         "    \"pools\": [\n"
         "        {\n"
-        "            \"comment\": \"foo\",\n"
         "            \"option-data\": [ ],\n"
         "            \"pool\": \"192.0.2.1-192.0.2.10\",\n"
-        "            \"user-context\": { \"version\": 1 }\n"
+        "            \"user-context\": { \"comment\": \"foo\",\n"
+        "                                \"version\": 1 }\n"
         "        },{\n"
         "            \"option-data\": [ ],\n"
         "            \"pool\": \"192.0.2.64/26\",\n"
