@@ -1,9 +1,11 @@
-// Copyright (C) 2013-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <hooks/callout_handle.h>
 #include <hooks/callout_manager.h>
 #include <hooks/callout_manager.h>
@@ -24,7 +26,8 @@ namespace hooks {
 
 // Constructor
 
-HooksManager::HooksManager() {
+HooksManager::HooksManager() : test_mode_(false) {
+    init();
 }
 
 // Return reference to singleton hooks manager.
@@ -39,7 +42,6 @@ HooksManager::getHooksManager() {
 
 bool
 HooksManager::calloutsPresentInternal(int index) {
-    conditionallyInitialize();
     return (callout_manager_->calloutsPresent(index));
 }
 
@@ -50,7 +52,6 @@ HooksManager::calloutsPresent(int index) {
 
 bool
 HooksManager::commandHandlersPresentInternal(const std::string& command_name) {
-    conditionallyInitialize();
     return (callout_manager_->commandHandlersPresent(command_name));
 }
 
@@ -63,7 +64,6 @@ HooksManager::commandHandlersPresent(const std::string& command_name) {
 
 void
 HooksManager::callCalloutsInternal(int index, CalloutHandle& handle) {
-    conditionallyInitialize();
     callout_manager_->callCallouts(index, handle);
 }
 
@@ -75,7 +75,6 @@ HooksManager::callCallouts(int index, CalloutHandle& handle) {
 void
 HooksManager::callCommandHandlersInternal(const std::string& command_name,
                                           CalloutHandle& handle) {
-    conditionallyInitialize();
     callout_manager_->callCommandHandlers(command_name, handle);
 }
 
@@ -85,14 +84,17 @@ HooksManager::callCommandHandlers(const std::string& command_name,
     getHooksManager().callCommandHandlersInternal(command_name, handle);
 }
 
-
 // Load the libraries.  This will delete the previously-loaded libraries
-// (if present) and load new ones.
+// (if present) and load new ones. If loading libraries fails, initialize with
+// empty list.
 
 bool
 HooksManager::loadLibrariesInternal(const HookLibsCollection& libraries) {
-    // Unload current set of libraries (if any are loaded).
-    unloadLibrariesInternal();
+    if (test_mode_) {
+        return (true);
+    }
+
+    ServerHooks::getServerHooks().getParkingLotsPtr()->clear();
 
     // Create the library manager and load the libraries.
     lm_collection_.reset(new LibraryManagerCollection(libraries));
@@ -104,8 +106,7 @@ HooksManager::loadLibrariesInternal(const HookLibsCollection& libraries) {
     } else {
         // Unable to load libraries, reset to state before this function was
         // called.
-        lm_collection_.reset();
-        callout_manager_.reset();
+        init();
     }
 
     return (status);
@@ -116,18 +117,14 @@ HooksManager::loadLibraries(const HookLibsCollection& libraries) {
     return (getHooksManager().loadLibrariesInternal(libraries));
 }
 
-// Unload the libraries.  This just deletes all internal objects which will
-// cause the libraries to be unloaded.
+// Unload the libraries.  This just deletes all internal objects (which will
+// cause the libraries to be unloaded) and initializes them with empty list if
+// requested.
 
 void
 HooksManager::unloadLibrariesInternal() {
-    // The order of deletion does not matter here, as each library manager
-    // holds its own pointer to the callout manager.  However, we may as
-    // well delete the library managers first: if there are no other references
-    // to the callout manager, the second statement will delete it, which may
-    // ease debugging.
-    lm_collection_.reset();
-    callout_manager_.reset();
+    ServerHooks::getServerHooks().getParkingLotsPtr()->clear();
+    init();
 }
 
 void HooksManager::unloadLibraries() {
@@ -138,9 +135,7 @@ void HooksManager::unloadLibraries() {
 
 boost::shared_ptr<CalloutHandle>
 HooksManager::createCalloutHandleInternal() {
-    conditionallyInitialize();
-    return (boost::shared_ptr<CalloutHandle>(
-            new CalloutHandle(callout_manager_, lm_collection_)));
+    return (boost::make_shared<CalloutHandle>(callout_manager_, lm_collection_));
 }
 
 boost::shared_ptr<CalloutHandle>
@@ -152,14 +147,12 @@ HooksManager::createCalloutHandle() {
 
 std::vector<std::string>
 HooksManager::getLibraryNamesInternal() const {
-    return (lm_collection_ ? lm_collection_->getLibraryNames()
-                           : std::vector<std::string>());
+    return (lm_collection_->getLibraryNames());
 }
 
 HookLibsCollection
 HooksManager::getLibraryInfoInternal() const {
-    return (lm_collection_ ? lm_collection_->getLibraryInfo()
-            : HookLibsCollection());
+    return (lm_collection_->getLibraryInfo());
 }
 
 std::vector<std::string>
@@ -172,17 +165,15 @@ HooksManager::getLibraryInfo() {
     return (getHooksManager().getLibraryInfoInternal());
 }
 
-// Perform conditional initialization if nothing is loaded.
+// Perform initialization
 
 void
-HooksManager::performConditionalInitialization() {
-
+HooksManager::init() {
     // Nothing present, so create the collection with any empty set of
     // libraries, and get the CalloutManager.
     HookLibsCollection libraries;
     lm_collection_.reset(new LibraryManagerCollection(libraries));
     lm_collection_->loadLibraries();
-
     callout_manager_ = lm_collection_->getCalloutManager();
 }
 
@@ -197,7 +188,6 @@ HooksManager::registerHook(const std::string& name) {
 
 isc::hooks::LibraryHandle&
 HooksManager::preCalloutsLibraryHandleInternal() {
-    conditionallyInitialize();
     return (callout_manager_->getPreLibraryHandle());
 }
 
@@ -208,7 +198,6 @@ HooksManager::preCalloutsLibraryHandle() {
 
 isc::hooks::LibraryHandle&
 HooksManager::postCalloutsLibraryHandleInternal() {
-    conditionallyInitialize();
     return (callout_manager_->getPostLibraryHandle());
 }
 
@@ -224,10 +213,14 @@ HooksManager::validateLibraries(const std::vector<std::string>& libraries) {
     return (LibraryManagerCollection::validateLibraries(libraries));
 }
 
-// Shared callout manager
-boost::shared_ptr<CalloutManager>&
-HooksManager::getSharedCalloutManager() {
-    return (getHooksManager().shared_callout_manager_);
+void
+HooksManager::setTestMode(bool mode) {
+    getHooksManager().test_mode_ = mode;
+}
+
+bool
+HooksManager::getTestMode() {
+    return (getHooksManager().test_mode_);
 }
 
 } // namespace util
