@@ -1,14 +1,17 @@
-// Copyright (C) 2013-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <hooks/callout_handle.h>
 #include <hooks/hooks_manager.h>
 #include <hooks/server_hooks.h>
 
 #include <hooks/tests/common_test_class.h>
+#define TEST_ASYNC_CALLOUT
 #include <hooks/tests/test_libraries.h>
 #include <cc/data.h>
 
@@ -35,15 +38,16 @@ public:
     /// Reset the hooks manager.  The hooks manager is a singleton, so needs
     /// to be reset for each test.
     HooksManagerTest() {
+        HooksManager::setTestMode(false);
         HooksManager::unloadLibraries();
     }
 
     /// @brief Destructor
     ///
-    /// Unload all libraries and reset the shared manager.
+    /// Unload all libraries.
     ~HooksManagerTest() {
+        HooksManager::setTestMode(false);
         HooksManager::unloadLibraries();
-        HooksManager::getSharedCalloutManager().reset();
     }
 
 
@@ -434,15 +438,15 @@ TEST_F(HooksManagerTest, PrePostCalloutTest) {
     EXPECT_EQ(-15, result);
 }
 
-// Test with a shared manager the pre- and post- callout functions survive
+// Test with test mode enabled and the pre- and post- callout functions survive
 // a reload
 
-TEST_F(HooksManagerTest, PrePostCalloutShared) {
+TEST_F(HooksManagerTest, TestModeEnabledPrePostSurviveLoad) {
 
+    // Load a single library.
     HookLibsCollection library_names;
-
-    // Initialize the shared manager.
-    HooksManager::getSharedCalloutManager().reset(new CalloutManager(0));
+    library_names.push_back(make_pair(std::string(FULL_CALLOUT_LIBRARY),
+                                      data::ConstElementPtr()));
 
     // Load the pre- and post- callouts.
     HooksManager::preCalloutsLibraryHandle().registerCallout("hookpt_two",
@@ -450,6 +454,8 @@ TEST_F(HooksManagerTest, PrePostCalloutShared) {
     HooksManager::postCalloutsLibraryHandle().registerCallout("hookpt_two",
                                                               testPostCallout);
 
+    HooksManager::setTestMode(true);
+
     // With the pre- and post- callouts above, the result expected is
     //
     // 1027 * 2
@@ -479,24 +485,24 @@ TEST_F(HooksManagerTest, PrePostCalloutShared) {
     EXPECT_EQ(2054, result);
 }
 
-// Test with a shared manager the pre- and post- callout functions survive
-// a reload but not with a not empty list of libraries
+// Test with test mode disabled and the pre- and post- callout functions do not
+// survive a reload
 
-TEST_F(HooksManagerTest, PrePostCalloutSharedNotEmpty) {
+TEST_F(HooksManagerTest, TestModeDisabledPrePostDoNotSurviveLoad) {
 
+    // Load a single library.
     HookLibsCollection library_names;
     library_names.push_back(make_pair(std::string(FULL_CALLOUT_LIBRARY),
                                       data::ConstElementPtr()));
 
-    // Initialize the shared manager.
-    HooksManager::getSharedCalloutManager().reset(new CalloutManager(0));
-
     // Load the pre- and post- callouts.
     HooksManager::preCalloutsLibraryHandle().registerCallout("hookpt_two",
                                                              testPreCallout);
     HooksManager::postCalloutsLibraryHandle().registerCallout("hookpt_two",
                                                               testPostCallout);
 
+    HooksManager::setTestMode(false);
+
     // With the pre- and post- callouts above, the result expected is
     //
     // 1027 * 2
@@ -511,7 +517,7 @@ TEST_F(HooksManagerTest, PrePostCalloutSharedNotEmpty) {
     EXPECT_EQ(2054, result);
 
     // ... and check that the pre- and post- callout functions don't survive a
-    // reload with a not empty list of libraries.
+    // reload.
     EXPECT_TRUE(HooksManager::loadLibraries(library_names));
     handle = HooksManager::createCalloutHandle();
 
@@ -520,22 +526,20 @@ TEST_F(HooksManagerTest, PrePostCalloutSharedNotEmpty) {
 
     HooksManager::callCallouts(hookpt_two_index_, *handle);
 
-    // Expect result - data_2
     result = 0;
     handle->getArgument("result", result);
     EXPECT_EQ(-15, result);
 }
 
-// Test with a shared manager the pre- and post- callout functions don't
-// survive a reload if the shared manager is initialized too late.
+// Test with test mode enabled and the pre- and post- callout functions do not
+// survive a reload if the test mode is set too late.
 
-TEST_F(HooksManagerTest, PrePostCalloutSharedTooLate) {
+TEST_F(HooksManagerTest, TestModeEnabledTooLatePrePostDoNotSurvive) {
 
+    // Load a single library.
     HookLibsCollection library_names;
-    EXPECT_TRUE(HooksManager::loadLibraries(library_names));
-
-    // Initialize the shared manager (after loadLibraries so too late)
-    HooksManager::getSharedCalloutManager().reset(new CalloutManager(0));
+    library_names.push_back(make_pair(std::string(FULL_CALLOUT_LIBRARY),
+                                      data::ConstElementPtr()));
 
     // Load the pre- and post- callouts.
     HooksManager::preCalloutsLibraryHandle().registerCallout("hookpt_two",
@@ -561,15 +565,16 @@ TEST_F(HooksManagerTest, PrePostCalloutSharedTooLate) {
     EXPECT_TRUE(HooksManager::loadLibraries(library_names));
     handle = HooksManager::createCalloutHandle();
 
+    HooksManager::setTestMode(true);
+
     handle->setArgument("result", static_cast<int>(0));
     handle->setArgument("data_2", static_cast<int>(15));
 
     HooksManager::callCallouts(hookpt_two_index_, *handle);
 
-    // Expect no change so result = 0
     result = 0;
     handle->getArgument("result", result);
-    EXPECT_EQ(0, result);
+    EXPECT_EQ(-15, result);
 }
 
 // Check that everything works even with no libraries loaded.  First that
@@ -753,5 +758,205 @@ TEST_F(HooksManagerTest, LibraryParameters) {
     EXPECT_NO_THROW(HooksManager::unloadLibraries());
 }
 
+// This test verifies that an object can be parked in two different
+// callouts and that it is unparked when the last callout calls the
+// unpark function.
+TEST_F(HooksManagerTest, Parking) {
+    // Load the same library twice. Both installed callouts will trigger
+    // asynchronous operation.
+    HookLibsCollection library_names;
+    library_names.push_back(make_pair(std::string(ASYNC_CALLOUT_LIBRARY),
+                                      data::ConstElementPtr()));
+    library_names.push_back(make_pair(std::string(ASYNC_CALLOUT_LIBRARY),
+                                      data::ConstElementPtr()));
+
+    // Load the libraries.
+    EXPECT_TRUE(HooksManager::loadLibraries(library_names));
+
+    CalloutHandlePtr handle = HooksManager::createCalloutHandle();
+
+    // We could be parked any object. Typically it will be a pointer to the
+    // packet. In this case, however, it is simpler to just use a string.
+    std::string parked_object = "foo";
+    handle->setArgument("parked_object", parked_object);
+
+    // Call both installed callouts.
+    HooksManager::callCallouts(hookpt_one_index_, *handle);
+
+    // This boolean value will be set to true when the packet gets unparked.
+    bool unparked = false;
+
+    // The callouts instruct us to park the object. We associated the callback
+    // function with the parked object, which sets "unparked" flag to true. We
+    // can later test the value of this flag to verify when exactly the packet
+    // got unparked.
+    ASSERT_NO_THROW(
+        HooksManager::park<std::string>("hookpt_one", "foo",
+        [&unparked] {
+            unparked = true;
+        })
+    );
+
+    // We have two callouts which should have returned pointers to the
+    // functions which we can call to siumulate completion of asynchronous
+    // tasks.
+    std::function<void()> unpark_trigger_func1;
+    handle->getArgument("unpark_trigger1", unpark_trigger_func1);
+    // Call the first function. It should cause the hook library to call the
+    // "unpark" function. However, the object should not be unparked yet,
+    // because the other callout hasn't completed its scheduled asynchronous
+    // operation (keeps a reference on the parked object).
+    unpark_trigger_func1();
+    EXPECT_FALSE(unparked);
+
+    // Call the second function. This should decrease the reference count to
+    // 0 and the packet should be unparked.
+    std::function<void()> unpark_trigger_func2;
+    handle->getArgument("unpark_trigger2", unpark_trigger_func2);
+    unpark_trigger_func2();
+    EXPECT_TRUE(unparked);
+
+    // Try unloading the libraries.
+    EXPECT_NO_THROW(HooksManager::unloadLibraries());
+}
+
+// This test verifies that the server can also unpark the packet.
+TEST_F(HooksManagerTest, ServerUnpark) {
+    // Load the same library twice. Both installed callouts will trigger
+    // asynchronous operation.
+    HookLibsCollection library_names;
+    library_names.push_back(make_pair(std::string(ASYNC_CALLOUT_LIBRARY),
+                                      data::ConstElementPtr()));
+    library_names.push_back(make_pair(std::string(ASYNC_CALLOUT_LIBRARY),
+                                      data::ConstElementPtr()));
+    // Load libraries.
+    EXPECT_TRUE(HooksManager::loadLibraries(library_names));
+
+    CalloutHandlePtr handle = HooksManager::createCalloutHandle();
+
+    // We could be parked any object. Typically it will be a pointer to the
+    // packet. In this case, however, it is simpler to just use a string.
+    std::string parked_object = "foo";
+    handle->setArgument("parked_object", parked_object);
+
+    // Call installed callout.
+    HooksManager::callCallouts(hookpt_one_index_, *handle);
+
+    // This boolean value will be set to true when the packet gets unparked.
+    bool unparked = false;
+
+    // It should be possible for the server to increase reference counter.
+    ASSERT_NO_THROW(HooksManager::reference<std::string>("hookpt_one", "foo"));
+
+    // The callouts instruct us to park the object. We associated the callback
+    // function with the parked object, which sets "unparked" flag to true. We
+    // can later test the value of this flag to verify when exactly the packet
+    // got unparked.
+    HooksManager::park<std::string>("hookpt_one", "foo",
+    [&unparked] {
+        unparked = true;
+    });
+
+    // Server can force unparking the object.
+    EXPECT_TRUE(HooksManager::unpark<std::string>("hookpt_one", "foo"));
+
+    EXPECT_TRUE(unparked);
+
+    // Try unloading the libraries.
+    EXPECT_NO_THROW(HooksManager::unloadLibraries());
+}
+
+// This test verifies that the server can drop parked packet.
+TEST_F(HooksManagerTest, ServerDropParked) {
+    // Load library.
+    HookLibsCollection library_names;
+    library_names.push_back(make_pair(std::string(ASYNC_CALLOUT_LIBRARY),
+                                      data::ConstElementPtr()));
+    EXPECT_TRUE(HooksManager::loadLibraries(library_names));
+
+    CalloutHandlePtr handle = HooksManager::createCalloutHandle();
+
+    // We could be parked any object. Typically it will be a pointer to the
+    // packet. In this case, however, it is simpler to just use a string.
+    std::string parked_object = "foo";
+    handle->setArgument("parked_object", parked_object);
+
+    // Call installed callout.
+    HooksManager::callCallouts(hookpt_one_index_, *handle);
+
+    // This boolean value will be set to true when the packet gets unparked.
+    bool unparked = false;
+
+    // It should be possible for the server to increase reference counter.
+    ASSERT_NO_THROW(HooksManager::reference<std::string>("hookpt_one", "foo"));
+
+    // The callouts instruct us to park the object. We associated the callback
+    // function with the parked object, which sets "unparked" flag to true. We
+    // can later test the value of this flag to verify when exactly the packet
+    // got unparked.
+    HooksManager::park<std::string>("hookpt_one", "foo",
+    [&unparked] {
+        unparked = true;
+    });
+
+    // Drop the parked packet. The callback should not be called.
+    EXPECT_TRUE(HooksManager::drop<std::string>("hookpt_one", "foo"));
+
+    EXPECT_FALSE(unparked);
+
+    // An attempt to unpark the packet should return false, as this packet
+    // is not parked anymore.
+    EXPECT_FALSE(HooksManager::unpark<std::string>("hookpt_one", "foo"));
+
+    // Try unloading the libraries.
+    EXPECT_NO_THROW(HooksManager::unloadLibraries());
+}
+
+// This test verifies that parked objects are removed when libraries are
+// unloaded.
+TEST_F(HooksManagerTest, UnloadBeforeUnpark) {
+    // Load the same library twice. Both installed callouts will trigger
+    // asynchronous operation.
+    HookLibsCollection library_names;
+    library_names.push_back(make_pair(std::string(ASYNC_CALLOUT_LIBRARY),
+                                      data::ConstElementPtr()));
+    library_names.push_back(make_pair(std::string(ASYNC_CALLOUT_LIBRARY),
+                                      data::ConstElementPtr()));
+    // Load libraries.
+    EXPECT_TRUE(HooksManager::loadLibraries(library_names));
+
+    CalloutHandlePtr handle = HooksManager::createCalloutHandle();
+
+    // We could be parked any object. Typically it will be a pointer to the
+    // packet. In this case, however, it is simpler to just use a string.
+    std::string parked_object = "foo";
+    handle->setArgument("parked_object", parked_object);
+
+    // Call installed callout.
+    HooksManager::callCallouts(hookpt_one_index_, *handle);
+
+    // This boolean value will be set to true when the packet gets unparked.
+    bool unparked = false;
+
+    // The callouts instruct us to park the object. We associated the callback
+    // function with the parked object, which sets "unparked" flag to true. We
+    // can later test the value of this flag to verify when exactly the packet
+    // got unparked.
+    HooksManager::park<std::string>("hookpt_one", "foo",
+    [&unparked] {
+        unparked = true;
+    });
+
+    // Try reloading the libraries.
+    EXPECT_NO_THROW(HooksManager::unloadLibraries());
+    EXPECT_TRUE(HooksManager::loadLibraries(library_names));
+
+    // Parked object should have been removed.
+    EXPECT_FALSE(HooksManager::unpark<std::string>("hookpt_one", "foo"));
+
+    // Callback should not be called.
+    EXPECT_FALSE(unparked);
+}
+
 
 } // Anonymous namespace
