@@ -111,7 +111,14 @@ DControllerBase::launch(int argc, char* argv[], const bool test_mode) {
 
     // Log the starting of the service.
     LOG_INFO(dctl_logger, DCTL_STARTING)
-        .arg(app_name_).arg(getpid()).arg(VERSION);
+        .arg(app_name_)
+        .arg(getpid())
+        .arg(VERSION)
+        .arg(PACKAGE_VERSION_TYPE);
+    // When it is not a stable version dissuade use in production.
+    if (std::string(PACKAGE_VERSION_TYPE) == "development") {
+        LOG_WARN(dctl_logger, DCTL_DEVELOPMENT_VERSION);
+    }
     try {
         // Step 2 is to create and initialize the application process object.
         initProcess();
@@ -811,6 +818,6 @@ DControllerBase::getVersion(bool extended) {
     return (tmp.str());
 }
 
-}; // namespace isc::process
+} // end of namespace isc::process
 
-}; // namespace isc
+} // end of namespace isc
