@@ -1,13 +1,13 @@
-// Copyright (C) 2013-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
+#include <kea_version.h>
 
 #include <cc/command_interpreter.h>
-#include <process/spec_config.h>
 #include <process/testutils/d_test_stubs.h>
 
 #include <boost/date_time/posix_time/posix_time.hpp>
@@ -52,9 +52,6 @@ TEST_F(DStubControllerTest, basicInstanceTesting) {
     // Verify that controller's bin name is correct.
     EXPECT_TRUE(checkBinName(DStubController::stub_bin_name_));
 
-    // Verify that controller's spec file name is correct.
-    EXPECT_TRUE(checkSpecFileName(D2_SPECFILE_LOCATION));
-
     // Verify that controller's IOService exists.
     EXPECT_TRUE(checkIOService());
 
@@ -331,12 +328,6 @@ TEST_F(DStubControllerTest, invalidConfigReload) {
     time_duration elapsed_time;
     runWithConfig("{ \"string_test\": \"first value\" }", 500, elapsed_time);
 
-    // Context is still available post launch. Check to see that our
-    // configuration value is still the original value.
-    std::string  actual_value = "";
-    ASSERT_NO_THROW(getContext()->getParam("string_test", actual_value));
-    EXPECT_EQ("first value", actual_value);
-
     // Verify that we saw the signal.
     std::vector<int>& signals = controller_->getProcessedSignals();
     ASSERT_EQ(1, signals.size());
@@ -358,12 +349,6 @@ TEST_F(DStubControllerTest, alternateParsing) {
     time_duration elapsed_time;
     runWithConfig("{ \"string_test\": \"first value\" }", 500, elapsed_time);
 
-    // Context is still available post launch. Check to see that our
-    // configuration value is still the original value.
-    std::string  actual_value = "";
-    ASSERT_NO_THROW(getContext()->getParam("string_test", actual_value));
-    EXPECT_EQ("alt value", actual_value);
-
     // Verify that we saw the signal.
     std::vector<int>& signals = controller_->getProcessedSignals();
     ASSERT_EQ(1, signals.size());
@@ -387,12 +372,6 @@ TEST_F(DStubControllerTest, validConfigReload) {
     time_duration elapsed_time;
     runWithConfig("{ \"string_test\": \"first value\" }", 800, elapsed_time);
 
-    // Context is still available post launch.
-    // Check to see that our configuration value is what we expect.
-    std::string  actual_value = "";
-    ASSERT_NO_THROW(getContext()->getParam("string_test", actual_value));
-    EXPECT_EQ("second value", actual_value);
-
     // Verify that we saw two occurrences of the signal.
     std::vector<int>& signals = controller_->getProcessedSignals();
     ASSERT_EQ(2, signals.size());
