@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -12,6 +12,7 @@
 #include <gtest/gtest.h>
 
 #include <unistd.h>
+#include <atomic>
 
 using namespace std;
 using namespace isc;
@@ -84,7 +85,7 @@ public:
 
     /// @brief Counter used to track the number of passes made
     /// within the thread worker function.
-    int passes_;
+    std::atomic<int> passes_;
 };
 
 const int WatchedThreadTest::WORKER_MAX_PASSES = 10;
